import { getToken, setToken, removeToken, getFirst, setFirst } from '@/utils/auth'
import { login, getInfo, logout } from '@/api/user'
import { resetRouter } from '@/router'
import utils from '@/utils/lib'

const state = {
  token: getToken(),
  first: getFirst(),
  avatar: '',
  introduction: '',
  id: '',
  name: '',
  order: 0,
  income: 0,
  roles: []
}

const mutations = {
  SET_TOKEN: (state, token) => {
    state.token = token
  },
  SET_ID: (state, id) => {
    state.id = id
  },
  SET_AVATAR: (state, avatar) => {
    state.avatar = avatar
  },
  SET_NAME: (state, name) => {
    state.name = name
  },
  SET_ORDER: (state, order) => {
    state.order = order
  },
  SET_INCOME: (state, income) => {
    state.income = income
  },
  SET_ROLES: (state, roles) => {
    state.roles = roles
  },
  SET_FIRST: (state, first) => {
    state.first = first
  }
}

const actions = {
  // user login
  login({ commit }, userInfo) {
    return new Promise((resolve, reject) => {
      login(userInfo).then(data => {
        commit('SET_TOKEN', data.token)
        setToken(data.token)
        resolve(data.super)
      }).catch(err => reject(err))
    })
  },

  // get user info
  getInfo({ commit }) {
    return new Promise(resolve => {
      getInfo().then(info => {
        commit('SET_ID', info.id)
        commit('SET_NAME', info.name)
        commit('SET_ORDER', info.order)
        commit('SET_INCOME', info.income)
        const roles = info.super ? ['super', 'admin'] : ['admin']
        commit('SET_ROLES', roles)
        commit('SET_AVATAR', `/img/${info.super ? 'super' : ''}admin.svg`)
        resolve({ roles })
      })
    })
  },

  // user logout
  logout({ commit, dispatch }) {
    return new Promise(resolve => {
      logout(state.token).then(() => {
        commit('SET_TOKEN', '')
        commit('SET_ROLES', [])
        removeToken()
        resetRouter()
        dispatch('tagsView/delAllViews', null, { root: true })
        utils.CloseSocket()
        utils.ConnectSocket()
        resolve()
      })
    })
  },

  // remove token
  isFirst({ commit }) {
    return new Promise(resolve => {
      const first = state.first
      commit('SET_FIRST', false)
      setFirst()
      resolve(first)
    })
  }
}

export default {
  namespaced: true,
  state,
  mutations,
  actions
}
